import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.util.*;
import lab.game.*;
import java.io.IOException;

public class Main{
    public static void main(String[] args){
        Chess newChess = new Chess();
        newChess.run();
    }
}

/**
 * A New Button class that has piece attribute
 * @author Omiid
 */
class newJButton extends JButton{
    private Piece piece;
    private char X;
    private int Y;

    public newJButton(){
        this.piece = null;
        this.setIcon(null);
    }

    /**
     * Make new button
     * @param X The X-Axis
     * @param Y The Y-Axis
     * @author Omiid
     */
    public newJButton(char X, int Y){
        this.piece = null;
        this.X = X;
        this.Y = Y;
        this.setIcon(null);
    }

    /**
     * Make new button
     * @param p The piece
     * @param X The X-Axis
     * @param Y The Y-Axis
     * @author Omiid
     */
    public newJButton(Piece p, char X, int Y){
        this.piece = p;
        this.X = X;
        this.Y = Y;
        Image img = null;
        try {
            img = ImageIO.read(Chess.class.getResource("resources/images/" + p.getName().toLowerCase() + ".png"));
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        this.setIcon(new ImageIcon(img));
    }

    public Piece getPiece(){
        return this.piece;
    }

    public void setPiece(Piece p){
        this.piece = p;
    }

    public char getNewX(){
        return this.X;
    }

    public int getNewY(){
        return this.Y;
    }
}

/**
 * Main Chess Class
 */
class Chess implements MouseListener{
    private Board board = null;
    private String color = "W";
    private boolean isSelected = false;
    private newJButton inMove = null;
    private ArrayList<newJButton> btns = new ArrayList<newJButton>();
    private ArrayList<newJButton> whiteLost = new ArrayList<>();
    private ArrayList<newJButton> blackLost = new ArrayList<>();
    private JLabel caption;
    /**
     * Initial Run
     */
    public void run(){
        JFrame frame = new JFrame("Chess Frame");
        frame.setExtendedState(java.awt.Frame.MAXIMIZED_BOTH);
        // Biggest Panel
        JPanel fullPanel = new JPanel(new BorderLayout());
        // Board Panel
        JPanel boardPanel = new JPanel(new GridLayout(8,8));
        ArrayList<Piece> whitePieces = new ArrayList<Piece>();
        ArrayList<Piece> blackPieces = new ArrayList<Piece>();
        for(int i = 1; i < 9; i++){
            for(char j = 'A'; j < 'I'; j++){
                newJButton btn = null;

                if( i == 8 ){
                    if( j == 'A' || j == 'H' ){

                        Rook newRook = new Rook(j, i, true, "WR");
                        whitePieces.add(newRook);
                        btn = new newJButton(newRook, j, i);
                    }else if( j == 'B' || j == 'G' ){

                        Knight newKnight = new Knight(j, i, true, "WN");
                        whitePieces.add(newKnight);
                        btn = new newJButton(newKnight, j, i);
                    }else if( j == 'C' || j == 'F' ){

                        Bishop newBishop = new Bishop(j, i, true, "WB");
                        whitePieces.add(newBishop);
                        btn = new newJButton(newBishop, j, i);
                    }else if( j == 'D' ){

                        Queen whiteQueen = new Queen(j, i, true,"WQ");
                        whitePieces.add(whiteQueen);
                        btn = new newJButton(whiteQueen, j, i);
                    }else if( j == 'E' ){
                        King whiteKing = new King(j, i, true, "WK");
                        whitePieces.add(whiteKing);
                        btn = new newJButton(whiteKing, j, i);
                    }
                }else if( i == 7 ){
                    Pawn newPawn = new Pawn(j, i, true, "WP");
                    whitePieces.add(newPawn);
                    btn = new newJButton(newPawn, j, i);
                }else if( i == 1 ){
                    if( j == 'A' || j == 'H' ){

                        Rook newRook = new Rook(j, i, false, "BR");
                        blackPieces.add(newRook);
                        btn = new newJButton(newRook, j, i);
                    }else if( j == 'B' || j == 'G' ){

                        Knight newKnight = new Knight(j, i, false, "BN");
                        blackPieces.add(newKnight);
                        btn = new newJButton(newKnight, j, i);
                    }else if( j == 'C' || j == 'F' ){

                        Bishop newBishop = new Bishop(j, i, false, "BB");
                        blackPieces.add(newBishop);
                        btn = new newJButton(newBishop, j, i);
                    }else if( j == 'D' ){

                        Queen blackQueen = new Queen(j, i, false,"BQ");
                        blackPieces.add(blackQueen);
                        btn = new newJButton(blackQueen, j, i);
                    }else if( j == 'E' ){
                        King blackKing = new King(j, i, false, "BQ");
                        blackPieces.add(blackKing);
                        btn = new newJButton(blackKing, j, i);
                    }
                }else if( i == 2 ){
                    Pawn newPawn = new Pawn(j, i, false, "BP");
                    blackPieces.add(newPawn);
                    btn = new newJButton(newPawn, j, i);
//                    btn = new newJButton(j, i);
                }else{
                    btn = new newJButton(j, i);
                }
                this.board = new Board(whitePieces, blackPieces);
                this.btns.add(btn);
                btn.addMouseListener(this);
                btn.setFocusable(false);
                if( (i % 2 == 1 && j % 2 == 1) || (i % 2 == 0 && j % 2 == 0) ){
                    btn.setBackground(new Color(219,217,164));
                }else{
                    btn.setBackground(new Color(4,51,106));
                }
                boardPanel.add(btn);
            }
        }
        // Left Divider Panel
        JPanel leftDivider = new JPanel(new GridLayout(3, 1));
        // Top Left
        JPanel topLeft = new JPanel(new GridLayout(2, 8));
        for(char i = 0; i < 2; i++){
            for(int j = 0; j < 8; j++){
                newJButton btn = new newJButton();
                ImageIcon icon = new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB));
                btn.setIcon(icon);
                btn.setFocusable(false);
                btn.setBackground(new Color(0,255,0));
                topLeft.add(btn);
                whiteLost.add(btn);
            }
        }
        // Middle Left
        JPanel middleLeft = new JPanel();
        JLabel playing = new JLabel("White Player is playing");
        caption = playing;
        middleLeft.add(playing);
        leftDivider.add(middleLeft);
        // Bottom Left
        JPanel bottomLeft = new JPanel(new GridLayout(2, 8));
        for(int i = 0; i < 2; i++){
            for(int j = 0; j < 8; j++){
                newJButton btn = new newJButton();
                ImageIcon icon = new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB));
                btn.setIcon(icon);
                btn.setFocusable(false);
                btn.setBackground(new Color(0,255,0));
                bottomLeft.add(btn);
                blackLost.add(btn);
            }
        }
        // Add Panels
        leftDivider.add(topLeft);
        leftDivider.add(middleLeft);
        leftDivider.add(bottomLeft);

        fullPanel.add(boardPanel, BorderLayout.EAST);
        fullPanel.add(leftDivider, BorderLayout.WEST);

        frame.add(fullPanel);
        // Show Frame
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * Mouse Clicking behaviour
     * @param e The event Happened
     */
    @Override
    public void mouseClicked(MouseEvent e) {
        newJButton source = (newJButton)(e.getSource());
        if( !isSelected ){
            for(newJButton btn : this.btns){
                if( source != btn && source.getPiece() != null && this.board.canGo(source.getPiece(), btn.getNewX(), btn.getNewY(), this.color) ){
                    if( this.board.canAttack(source.getPiece(), btn.getNewX(), btn.getNewY(), this.color) ){

                        btn.setBackground(new Color(255, 0, 0));
                    }else{
                        Image img = null;
                        try {
                            img = ImageIO.read(Chess.class.getResource("resources/images/dot.png"));
                        } catch (IOException e1) {
                            e1.printStackTrace();
                        }
                        btn.setIcon(new ImageIcon(img, "dot"));
                    }
                    this.isSelected = true;
                    this.inMove = source;
                }
            }
        }else{
            if( this.board.canGo(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){
                if( this.board.canAttack(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){

                    Piece temp = source.getPiece();
                    source.setPiece(this.inMove.getPiece());
                    this.inMove.setPiece(null);
                    if( this.color.equals("W") ){

                        killBlack(temp);
                    }else{
                        killWhite(temp);
                    }
                }else{
                    Piece temp = this.inMove.getPiece();
                    this.inMove.setPiece(source.getPiece());
                    source.setPiece(temp);
                }
                System.out.println(this.board.move( inMove.getNewY() + Character.toString(inMove.getNewX()), source.getNewY() + Character.toString(source.getNewX()), this.color));
                if( this.color.equals("W")){

                    caption.setText("Black Player is playing");
                    this.color = "B";
                }else{
                    caption.setText("White Player is playing");
                    this.color = "W";
                }
            }
            for(newJButton btn : this.btns) {
//                if( ((ImageIcon)btn.getIcon()) != null ){
//                    System.out.println(((ImageIcon)btn.getIcon()).getDescription());
//                }
                if( btn.getPiece() != null ){
                    Image img = null;
                    try {
                        img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    btn.setIcon(new ImageIcon(img));
                }else{
                    btn.setIcon(null);
                }
                if( (btn.getNewY() % 2 == 1 && btn.getNewX() % 2 == 1) || (btn.getNewY() % 2 == 0 && btn.getNewX() % 2 == 0) ){
                    btn.setBackground(new Color(219,217,164));
                }else{
                    btn.setBackground(new Color(4,51,106));
                }
                this.isSelected = false;
                this.inMove = null;
            }
            for(newJButton btn : this.whiteLost) {
                if( btn.getPiece() != null ){
                    Image img = new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB);
                    try {
                        img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    btn.setIcon(new ImageIcon(img));
                }else{
                    btn.setIcon(new ImageIcon(
                            new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB)));
                }
//                if( (btn.getNewY() % 2 == 1 && btn.getNewX() % 2 == 1) || (btn.getNewY() % 2 == 0 && btn.getNewX() % 2 == 0) ){
//                    btn.setBackground(new Color(219,217,164));
//                }else{
//                    btn.setBackground(new Color(4,51,106));
//                }
//                this.isSelected = false;
//                this.inMove = null;
            }
            for(newJButton btn : this.blackLost) {
                if( btn.getPiece() != null ){
                    Image img = new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB);
                    try {
                        img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    btn.setIcon(new ImageIcon(img));
                }else{
                    btn.setIcon(new ImageIcon(
                            new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB)));
                }
//                if( (btn.getNewY() % 2 == 1 && btn.getNewX() % 2 == 1) || (btn.getNewY() % 2 == 0 && btn.getNewX() % 2 == 0) ){
//                    btn.setBackground(new Color(219,217,164));
//                }else{
//                    btn.setBackground(new Color(4,51,106));
//                }
//                this.isSelected = false;
//                this.inMove = null;
            }
        }
    }

    public boolean killWhite(Piece p){
        for(newJButton btn : this.whiteLost){
            if( btn.getPiece() == null ){

                btn.setPiece(p);
                return true;
            }
        }
        return false;
    }

    public boolean killBlack(Piece p){
        for(newJButton btn : this.blackLost){
            if( btn.getPiece() == null ){

                btn.setPiece(p);
                return true;
            }
        }
        return false;
    }

    @Override
    public void mousePressed(MouseEvent e) {

    }

    @Override
    public void mouseReleased(MouseEvent e) {

    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {

    }
}
